<?xml version="1.0" encoding="UTF-8" ?>
<gui name="killbots"
     version="3"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

	<MenuBar>
		<Menu name="moveMenu">
			<text>Move</text>
			<Action name="teleport"/>
			<Action name="teleport_safely"/>
			<Action name="teleport_sip"/>
			<Action name="vaporizer"/>
			<Action name="wait_out_round"/>
		</Menu>
	</MenuBar>

	<ToolBar name="mainToolBar">
		<text>Main Toolbar</text>
		<Action name="game_new"/>
		<Separator/>
		<Action name="game_highscores"/>
		<Separator/>
		<Action name="teleport"/>
		<Action name="teleport_safely"/>
		<Action name="wait_out_round"/>
		<Separator/>
		<Action name="options_configure"/>
	</ToolBar>

</gui>
